<?php

declare(strict_types=1);

namespace Drupal\nva\Plugin\Block;

use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Stinis\Nva\APIClient;

/**
 * Provides a list of publications by category.
 */
#[Block(
  id: 'nva_publications_by_category',
  admin_label: new TranslatableMarkup('List publications by category'),
  category: new TranslatableMarkup('NVA'),
)]
final class PublicationsByCategoryBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The NVA API client.
   *
   * @var \Stinis\Nva\APIClient
   */
  private APIClient $apiClient;

  public function __construct(array $configuration, $plugin_id, array $plugin_definition, APIClient $apiClient) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->apiClient = $apiClient;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new PublicationsByCategoryBlock(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('nva.api_client'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form['authorName'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Author Name'),
      '#default_value' => $this->configuration['authorName'] ?? '',
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['authorName'] = $form_state->getValue('authorName');
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $authorName = $variables['authorName'] ?? $this->configuration['authorName'] ?? '';
    // Gets the person ID from the API.
    $person = $this->apiClient->getCristinPersonByQueryParameters(['name' => $authorName]);
    if (isset($person['hits'][0])) {
      // If the person is found, get the person ID.
      $personID = $person['hits'][0]['id'];
      $queryParameters = [
        'contributor' => $personID,
        'results' => 10000,
      ];
      // Get publications from the API.
      $results = $this->apiClient->searchPublications($queryParameters);
      $publications = $results['hits'];
      //group by ['entityDescription']['reference']['publicationInstance']['type'].
      $publications = array_reduce($publications, static function ($carry, $item) {
        $type = $item['entityDescription']['reference']['publicationInstance']['type'];
        // Add a space before the second uppercase character in $type and make the second one lowercase.
        $type = ucfirst(strtolower(preg_replace('/([A-Z])/', ' $1', lcfirst($type))));
        $carry[$type][] = $item;
        return $carry;
      }, []);
    }

    $build['content'] = [
      '#theme' => 'nva_publications_by_category',
      '#publications' => $publications ?? [],
    ];
    return $build;
  }

}
